@@ -1,6 +1,6 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
+   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | Copyright (c) 1997-2016 The PHP Group                                |
    +----------------------------------------------------------------------+
@@ -54,16 +54,6 @@
 #include <limits.h>
 #endif
 
-#ifdef PHP_WIN32
-# include "win32/php_stdint.h"
-#else
-# if HAVE_INTTYPES_H
-#  include <inttypes.h>
-# elif HAVE_STDINT_H
-#  include <stdint.h>
-# endif
-#endif
-
 static int cmd_max_len;
 
 /* {{{ PHP_MINIT_FUNCTION(exec) */
@@ -73,11 +63,11 @@ PHP_MINIT_FUNCTION(exec)
 	cmd_max_len = sysconf(_SC_ARG_MAX);
 	if (-1 == cmd_max_len) {
 #ifdef _POSIX_ARG_MAX
-               cmd_max_len = _POSIX_ARG_MAX;
+		cmd_max_len = _POSIX_ARG_MAX;
 #else
-               cmd_max_len = 4096;
+		cmd_max_len = 4096;
 #endif
-       }
+	}
 #elif defined(ARG_MAX)
 	cmd_max_len = ARG_MAX;
 #elif defined(PHP_WIN32)
@@ -100,11 +90,12 @@ PHP_MINIT_FUNCTION(exec)
  * If type==3, output will be printed binary, no lines will be saved or returned (passthru)
  *
  */
-PHPAPI int php_exec(int type, char *cmd, zval *array, zval *return_value TSRMLS_DC)
+PHPAPI int php_exec(int type, char *cmd, zval *array, zval *return_value)
 {
 	FILE *fp;
 	char *buf;
-	int l = 0, pclose_return;
+	size_t l = 0;
+	int pclose_return;
 	char *b, *d=NULL;
 	php_stream *stream;
 	size_t buflen, bufl = 0;
@@ -122,7 +113,7 @@ PHPAPI int php_exec(int type, char *cmd, zval *array, zval *return_value TSRMLS_
 	fp = VCWD_POPEN(cmd, "r");
 #endif
 	if (!fp) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to fork [%s]", cmd);
+		php_error_docref(NULL, E_WARNING, "Unable to fork [%s]", cmd);
 		goto err;
 	}
 
@@ -152,18 +143,18 @@ PHPAPI int php_exec(int type, char *cmd, zval *array, zval *return_value TSRMLS_
 
 			if (type == 1) {
 				PHPWRITE(buf, bufl);
-				if (php_output_get_level(TSRMLS_C) < 1) {
-					sapi_flush(TSRMLS_C);
+				if (php_output_get_level() < 1) {
+					sapi_flush();
 				}
 			} else if (type == 2) {
 				/* strip trailing whitespaces */
 				l = bufl;
-				while (l-- && isspace(((unsigned char *)buf)[l]));
-				if (l != (int)(bufl - 1)) {
+				while (l-- > 0 && isspace(((unsigned char *)buf)[l]));
+				if (l != (bufl - 1)) {
 					bufl = l + 1;
 					buf[bufl] = '\0';
 				}
-				add_next_index_stringl(array, buf, bufl, 1);
+				add_next_index_stringl(array, buf, bufl);
 			}
 			b = buf;
 		}
@@ -171,18 +162,18 @@ PHPAPI int php_exec(int type, char *cmd, zval *array, zval *return_value TSRMLS_
 			/* strip trailing whitespaces if we have not done so already */
 			if ((type == 2 && buf != b) || type != 2) {
 				l = bufl;
-				while (l-- && isspace(((unsigned char *)buf)[l]));
-				if (l != (int)(bufl - 1)) {
+				while (l-- > 0 && isspace(((unsigned char *)buf)[l]));
+				if (l != (bufl - 1)) {
 					bufl = l + 1;
 					buf[bufl] = '\0';
 				}
 				if (type == 2) {
-					add_next_index_stringl(array, buf, bufl, 1);
+					add_next_index_stringl(array, buf, bufl);
 				}
 			}
 
 			/* Return last line from the shell command */
-			RETVAL_STRINGL(buf, bufl, 1);
+			RETVAL_STRINGL(buf, bufl);
 		} else { /* should return NULL, but for BC we return "" */
 			RETVAL_EMPTY_STRING();
 		}
@@ -214,36 +205,36 @@ err:
 static void php_exec_ex(INTERNAL_FUNCTION_PARAMETERS, int mode) /* {{{ */
 {
 	char *cmd;
-	int cmd_len;
+	size_t cmd_len;
 	zval *ret_code=NULL, *ret_array=NULL;
 	int ret;
 
 	if (mode) {
-		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|z/", &cmd, &cmd_len, &ret_code) == FAILURE) {
+		if (zend_parse_parameters(ZEND_NUM_ARGS(), "s|z/", &cmd, &cmd_len, &ret_code) == FAILURE) {
 			RETURN_FALSE;
 		}
 	} else {
-		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|z/z/", &cmd, &cmd_len, &ret_array, &ret_code) == FAILURE) {
+		if (zend_parse_parameters(ZEND_NUM_ARGS(), "s|z/z/", &cmd, &cmd_len, &ret_array, &ret_code) == FAILURE) {
 			RETURN_FALSE;
 		}
 	}
 	if (!cmd_len) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot execute a blank command");
+		php_error_docref(NULL, E_WARNING, "Cannot execute a blank command");
 		RETURN_FALSE;
 	}
 	if (strlen(cmd) != cmd_len) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "NULL byte detected. Possible attack");
+		php_error_docref(NULL, E_WARNING, "NULL byte detected. Possible attack");
 		RETURN_FALSE;
 	}
 
 	if (!ret_array) {
-		ret = php_exec(mode, cmd, NULL, return_value TSRMLS_CC);
+		ret = php_exec(mode, cmd, NULL, return_value);
 	} else {
 		if (Z_TYPE_P(ret_array) != IS_ARRAY) {
 			zval_dtor(ret_array);
 			array_init(ret_array);
 		}
-		ret = php_exec(2, cmd, ret_array, return_value TSRMLS_CC);
+		ret = php_exec(2, cmd, ret_array, return_value);
 	}
 	if (ret_code) {
 		zval_dtor(ret_code);
@@ -285,23 +276,23 @@ PHP_FUNCTION(passthru)
 
    *NOT* safe for binary strings
 */
-PHPAPI char *php_escape_shell_cmd(char *str)
+PHPAPI zend_string *php_escape_shell_cmd(char *str)
 {
 	register int x, y;
 	size_t l = strlen(str);
 	uint64_t estimate = (2 * (uint64_t)l) + 1;
-	char *cmd;
+	zend_string *cmd;
+#ifndef PHP_WIN32
 	char *p = NULL;
-
-	TSRMLS_FETCH();
+#endif
 
 	/* max command line length - two single quotes - \0 byte length */
 	if (l > cmd_max_len - 2 - 1) {
-		php_error_docref(NULL TSRMLS_CC, E_ERROR, "Command exceeds the allowed length of %d bytes", cmd_max_len);
-		return NULL;
+		php_error_docref(NULL, E_ERROR, "Command exceeds the allowed length of %d bytes", cmd_max_len);
+		return ZSTR_EMPTY_ALLOC();
 	}
 
-	cmd = safe_emalloc(2, l, 1);
+	cmd = zend_string_safe_alloc(2, l, 0, 0);
 
 	for (x = 0, y = 0; x < l; x++) {
 		int mb_len = php_mblen(str + x, (l - x));
@@ -310,7 +301,7 @@ PHPAPI char *php_escape_shell_cmd(char *str)
 		if (mb_len < 0) {
 			continue;
 		} else if (mb_len > 1) {
-			memcpy(cmd + y, str + x, mb_len);
+			memcpy(ZSTR_VAL(cmd) + y, str + x, mb_len);
 			y += mb_len;
 			x += mb_len - 1;
 			continue;
@@ -325,13 +316,13 @@ PHPAPI char *php_escape_shell_cmd(char *str)
 				} else if (p && *p == str[x]) {
 					p = NULL;
 				} else {
-					cmd[y++] = '\\';
+					ZSTR_VAL(cmd)[y++] = '\\';
 				}
-				cmd[y++] = str[x];
+				ZSTR_VAL(cmd)[y++] = str[x];
 				break;
 #else
-			/* % is Windows specific for enviromental variables, ^%PATH% will
-				output PATH while ^%PATH^% will not. escapeshellcmd will escape all % and !.
+			/* % is Windows specific for environmental variables, ^%PATH% will
+				output PATH while ^%PATH^% will not. escapeshellcmd->val will escape all % and !.
 			*/
 			case '%':
 			case '!':
@@ -360,57 +351,57 @@ PHPAPI char *php_escape_shell_cmd(char *str)
 			case '\x0A': /* excluding these two */
 			case '\xFF':
 #ifdef PHP_WIN32
-				cmd[y++] = '^';
+				ZSTR_VAL(cmd)[y++] = '^';
 #else
-				cmd[y++] = '\\';
+				ZSTR_VAL(cmd)[y++] = '\\';
 #endif
 				/* fall-through */
 			default:
-				cmd[y++] = str[x];
+				ZSTR_VAL(cmd)[y++] = str[x];
 
 		}
 	}
-	cmd[y] = '\0';
+	ZSTR_VAL(cmd)[y] = '\0';
 
 	if (y > cmd_max_len + 1) {
-		php_error_docref(NULL TSRMLS_CC, E_ERROR, "Escaped command exceeds the allowed length of %d bytes", cmd_max_len);
-		efree(cmd);
-		return NULL;
+		php_error_docref(NULL, E_ERROR, "Escaped command exceeds the allowed length of %d bytes", cmd_max_len);
+		zend_string_release(cmd);
+		return ZSTR_EMPTY_ALLOC();
 	}
 
 	if ((estimate - y) > 4096) {
 		/* realloc if the estimate was way overill
 		 * Arbitrary cutoff point of 4096 */
-		cmd = erealloc(cmd, y + 1);
+		cmd = zend_string_truncate(cmd, y, 0);
 	}
 
+	ZSTR_LEN(cmd) = y;
+
 	return cmd;
 }
 /* }}} */
 
 /* {{{ php_escape_shell_arg
  */
-PHPAPI char *php_escape_shell_arg(char *str)
+PHPAPI zend_string *php_escape_shell_arg(char *str)
 {
 	int x, y = 0;
 	size_t l = strlen(str);
-	char *cmd;
+	zend_string *cmd;
 	uint64_t estimate = (4 * (uint64_t)l) + 3;
 
-	TSRMLS_FETCH();
-
 	/* max command line length - two single quotes - \0 byte length */
 	if (l > cmd_max_len - 2 - 1) {
-		php_error_docref(NULL TSRMLS_CC, E_ERROR, "Argument exceeds the allowed length of %d bytes", cmd_max_len);
-		return NULL;
+		php_error_docref(NULL, E_ERROR, "Argument exceeds the allowed length of %d bytes", cmd_max_len);
+		return ZSTR_EMPTY_ALLOC();
 	}
 
-	cmd = safe_emalloc(4, l, 3); /* worst case */
+	cmd = zend_string_safe_alloc(4, l, 2, 0); /* worst case */
 
 #ifdef PHP_WIN32
-	cmd[y++] = '"';
+	ZSTR_VAL(cmd)[y++] = '"';
 #else
-	cmd[y++] = '\'';
+	ZSTR_VAL(cmd)[y++] = '\'';
 #endif
 
 	for (x = 0; x < l; x++) {
@@ -420,7 +411,7 @@ PHPAPI char *php_escape_shell_arg(char *str)
 		if (mb_len < 0) {
 			continue;
 		} else if (mb_len > 1) {
-			memcpy(cmd + y, str + x, mb_len);
+			memcpy(ZSTR_VAL(cmd) + y, str + x, mb_len);
 			y += mb_len;
 			x += mb_len - 1;
 			continue;
@@ -431,45 +422,46 @@ PHPAPI char *php_escape_shell_arg(char *str)
 		case '"':
 		case '%':
 		case '!':
-			cmd[y++] = ' ';
+			ZSTR_VAL(cmd)[y++] = ' ';
 			break;
 #else
 		case '\'':
-			cmd[y++] = '\'';
-			cmd[y++] = '\\';
-			cmd[y++] = '\'';
+			ZSTR_VAL(cmd)[y++] = '\'';
+			ZSTR_VAL(cmd)[y++] = '\\';
+			ZSTR_VAL(cmd)[y++] = '\'';
 #endif
 			/* fall-through */
 		default:
-			cmd[y++] = str[x];
+			ZSTR_VAL(cmd)[y++] = str[x];
 		}
 	}
 #ifdef PHP_WIN32
-	if (y > 0 && '\\' == cmd[y - 1]) {
+	if (y > 0 && '\\' == ZSTR_VAL(cmd)[y - 1]) {
 		int k = 0, n = y - 1;
-		for (; n >= 0 && '\\' == cmd[n]; n--, k++);
+		for (; n >= 0 && '\\' == ZSTR_VAL(cmd)[n]; n--, k++);
 		if (k % 2) {
-			cmd[y++] = '\\';
+			ZSTR_VAL(cmd)[y++] = '\\';
 		}
 	}
 
-	cmd[y++] = '"';
+	ZSTR_VAL(cmd)[y++] = '"';
 #else
-	cmd[y++] = '\'';
+	ZSTR_VAL(cmd)[y++] = '\'';
 #endif
-	cmd[y] = '\0';
+	ZSTR_VAL(cmd)[y] = '\0';
 
 	if (y > cmd_max_len + 1) {
-		php_error_docref(NULL TSRMLS_CC, E_ERROR, "Escaped argument exceeds the allowed length of %d bytes", cmd_max_len);
-		efree(cmd);
-		return NULL;
+		php_error_docref(NULL, E_ERROR, "Escaped argument exceeds the allowed length of %d bytes", cmd_max_len);
+		zend_string_release(cmd);
+		return ZSTR_EMPTY_ALLOC();
 	}
 
 	if ((estimate - y) > 4096) {
 		/* realloc if the estimate was way overill
 		 * Arbitrary cutoff point of 4096 */
-		cmd = erealloc(cmd, y + 1);
+		cmd = zend_string_truncate(cmd, y, 0);
 	}
+	ZSTR_LEN(cmd) = y;
 	return cmd;
 }
 /* }}} */
@@ -479,20 +471,18 @@ PHPAPI char *php_escape_shell_arg(char *str)
 PHP_FUNCTION(escapeshellcmd)
 {
 	char *command;
-	int command_len;
-	char *cmd = NULL;
+	size_t command_len;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &command, &command_len) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &command, &command_len) == FAILURE) {
 		return;
 	}
 
 	if (command_len) {
 		if (command_len != strlen(command)) {
-			php_error_docref(NULL TSRMLS_CC, E_ERROR, "Input string contains NULL bytes");
+			php_error_docref(NULL, E_ERROR, "Input string contains NULL bytes");
 			return;
 		}
-		cmd = php_escape_shell_cmd(command);
-		RETVAL_STRINGL_CHECK(cmd, strlen(cmd), 0);
+		RETVAL_STR(php_escape_shell_cmd(command));
 	} else {
 		RETVAL_EMPTY_STRING();
 	}
@@ -504,20 +494,18 @@ PHP_FUNCTION(escapeshellcmd)
 PHP_FUNCTION(escapeshellarg)
 {
 	char *argument;
-	int argument_len;
-	char *cmd = NULL;
+	size_t argument_len;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &argument, &argument_len) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &argument, &argument_len) == FAILURE) {
 		return;
 	}
 
 	if (argument) {
 		if (argument_len != strlen(argument)) {
-			php_error_docref(NULL TSRMLS_CC, E_ERROR, "Input string contains NULL bytes");
+			php_error_docref(NULL, E_ERROR, "Input string contains NULL bytes");
 			return;
 		}
-		cmd = php_escape_shell_arg(argument);
-		RETVAL_STRINGL_CHECK(cmd, strlen(cmd), 0);
+		RETVAL_STR(php_escape_shell_arg(argument));
 	}
 }
 /* }}} */
@@ -527,13 +515,12 @@ PHP_FUNCTION(escapeshellarg)
 PHP_FUNCTION(shell_exec)
 {
 	FILE *in;
-	size_t total_readbytes;
 	char *command;
-	int command_len;
-	char *ret;
+	size_t command_len;
+	zend_string *ret;
 	php_stream *stream;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &command, &command_len) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &command, &command_len) == FAILURE) {
 		return;
 	}
 
@@ -542,16 +529,16 @@ PHP_FUNCTION(shell_exec)
 #else
 	if ((in=VCWD_POPEN(command, "r"))==NULL) {
 #endif
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to execute '%s'", command);
+		php_error_docref(NULL, E_WARNING, "Unable to execute '%s'", command);
 		RETURN_FALSE;
 	}
 
 	stream = php_stream_fopen_from_pipe(in, "rb");
-	total_readbytes = php_stream_copy_to_mem(stream, &ret, PHP_STREAM_COPY_ALL, 0);
+	ret = php_stream_copy_to_mem(stream, PHP_STREAM_COPY_ALL, 0);
 	php_stream_close(stream);
 
-	if (total_readbytes > 0) {
-		RETVAL_STRINGL_CHECK(ret, total_readbytes, 0);
+	if (ret && ZSTR_LEN(ret) > 0) {
+		RETVAL_STR(ret);
 	}
 }
 /* }}} */
@@ -561,16 +548,16 @@ PHP_FUNCTION(shell_exec)
    Change the priority of the current process */
 PHP_FUNCTION(proc_nice)
 {
-	long pri;
+	zend_long pri;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", &pri) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "l", &pri) == FAILURE) {
 		RETURN_FALSE;
 	}
 
 	errno = 0;
 	php_ignore_value(nice(pri));
 	if (errno) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Only a super user may attempt to increase the priority of a process");
+		php_error_docref(NULL, E_WARNING, "Only a super user may attempt to increase the priority of a process");
 		RETURN_FALSE;
 	}
 
