@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
+   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2013 The PHP Group                                |
+   | Copyright (c) 1997-2016 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -24,7 +24,7 @@
 #include "php.h"
 
 #include <stddef.h>
-
+#include <errno.h>
 
 
 #ifdef PHP_WIN32
@@ -166,7 +166,7 @@ PHPAPI void php_network_freeaddresses(struct sockaddr **sal)
 /* {{{ php_network_getaddresses
  * Returns number of addresses, 0 for none/error
  */
-PHPAPI int php_network_getaddresses(const char *host, int socktype, struct sockaddr ***sal, char **error_string TSRMLS_DC)
+PHPAPI int php_network_getaddresses(const char *host, int socktype, struct sockaddr ***sal, zend_string **error_string)
 {
 	struct sockaddr **sap;
 	int n;
@@ -212,18 +212,18 @@ PHPAPI int php_network_getaddresses(const char *host, int socktype, struct socka
 
 	if ((n = getaddrinfo(host, NULL, &hints, &res))) {
 		if (error_string) {
-			spprintf(error_string, 0, "php_network_getaddresses: getaddrinfo failed: %s", PHP_GAI_STRERROR(n));
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", *error_string);
+			*error_string = strpprintf(0, "php_network_getaddresses: getaddrinfo failed: %s", PHP_GAI_STRERROR(n));
+			php_error_docref(NULL, E_WARNING, "%s", ZSTR_VAL(*error_string));
 		} else {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "php_network_getaddresses: getaddrinfo failed: %s", PHP_GAI_STRERROR(n));
+			php_error_docref(NULL, E_WARNING, "php_network_getaddresses: getaddrinfo failed: %s", PHP_GAI_STRERROR(n));
 		}
 		return 0;
 	} else if (res == NULL) {
 		if (error_string) {
-			spprintf(error_string, 0, "php_network_getaddresses: getaddrinfo failed (null result pointer) errno=%d", errno);
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", *error_string);
+			*error_string = strpprintf(0, "php_network_getaddresses: getaddrinfo failed (null result pointer) errno=%d", errno);
+			php_error_docref(NULL, E_WARNING, "%s", ZSTR_VAL(*error_string));
 		} else {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "php_network_getaddresses: getaddrinfo failed (null result pointer)");
+			php_error_docref(NULL, E_WARNING, "php_network_getaddresses: getaddrinfo failed (null result pointer)");
 		}
 		return 0;
 	}
@@ -245,14 +245,18 @@ PHPAPI int php_network_getaddresses(const char *host, int socktype, struct socka
 	freeaddrinfo(res);
 #else
 	if (!inet_aton(host, &in)) {
-		/* XXX NOT THREAD SAFE (is safe under win32) */
-		host_info = gethostbyname(host);
+		if(strlen(host) > MAXFQDNLEN) {
+			host_info = NULL;
+			errno = E2BIG;
+		} else {
+			host_info = php_network_gethostbyname(host);
+		}
 		if (host_info == NULL) {
 			if (error_string) {
-				spprintf(error_string, 0, "php_network_getaddresses: gethostbyname failed. errno=%d", errno);
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "%s", *error_string);
+				*error_string = strpprintf(0, "php_network_getaddresses: gethostbyname failed. errno=%d", errno);
+				php_error_docref(NULL, E_WARNING, "%s", ZSTR_VAL(*error_string));
 			} else {
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "php_network_getaddresses: gethostbyname failed");
+				php_error_docref(NULL, E_WARNING, "php_network_getaddresses: gethostbyname failed");
 			}
 			return 0;
 		}
@@ -305,7 +309,7 @@ PHPAPI int php_network_connect_socket(php_socket_t sockfd,
 		socklen_t addrlen,
 		int asynchronous,
 		struct timeval *timeout,
-		char **error_string,
+		zend_string **error_string,
 		int *error_code)
 {
 #if HAVE_NON_BLOCKING_CONNECT
@@ -326,7 +330,7 @@ PHPAPI int php_network_connect_socket(php_socket_t sockfd,
 
 		if (error != EINPROGRESS) {
 			if (error_string) {
-				*error_string = php_socket_strerror(error, NULL, 0);
+				*error_string = php_socket_error_str(error);
 			}
 
 			return -1;
@@ -382,13 +386,13 @@ ok:
 	if (error) {
 		ret = -1;
 		if (error_string) {
-			*error_string = php_socket_strerror(error, NULL, 0);
+			*error_string = php_socket_error_str(error);
 		}
 	}
 	return ret;
 #else
 	if (asynchronous) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Asynchronous connect() not supported on this platform");
+		php_error_docref(NULL, E_WARNING, "Asynchronous connect() not supported on this platform");
 	}
 	return (connect(sockfd, addr, addrlen) == 0) ? 0 : -1;
 #endif
@@ -416,15 +420,16 @@ static inline void sub_times(struct timeval a, struct timeval b, struct timeval
  * */
 /* {{{ php_network_bind_socket_to_local_addr */
 php_socket_t php_network_bind_socket_to_local_addr(const char *host, unsigned port,
-		int socktype, char **error_string, int *error_code
-		TSRMLS_DC)
+		int socktype, long sockopts, zend_string **error_string, int *error_code
+		)
 {
 	int num_addrs, n, err = 0;
 	php_socket_t sock;
 	struct sockaddr **sal, **psal, *sa;
 	socklen_t socklen;
+	int sockoptval = 1;
 
-	num_addrs = php_network_getaddresses(host, socktype, &psal, error_string TSRMLS_CC);
+	num_addrs = php_network_getaddresses(host, socktype, &psal, error_string);
 
 	if (num_addrs == 0) {
 		/* could not resolve address(es) */
@@ -464,9 +469,22 @@ php_socket_t php_network_bind_socket_to_local_addr(const char *host, unsigned po
 			/* attempt to bind */
 
 #ifdef SO_REUSEADDR
-			{
-				int val = 1;
-				setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char*)&val, sizeof(val));
+			setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char*)&sockoptval, sizeof(sockoptval));
+#endif
+#ifdef IPV6_V6ONLY
+			if (sockopts & STREAM_SOCKOP_IPV6_V6ONLY) {
+				int ipv6_val = !!(sockopts & STREAM_SOCKOP_IPV6_V6ONLY_ENABLED);
+				setsockopt(sock, IPPROTO_IPV6, IPV6_V6ONLY, (char*)&ipv6_val, sizeof(sockoptval));
+			}
+#endif
+#ifdef SO_REUSEPORT
+			if (sockopts & STREAM_SOCKOP_SO_REUSEPORT) {
+				setsockopt(sock, SOL_SOCKET, SO_REUSEPORT, (char*)&sockoptval, sizeof(sockoptval));
+			}
+#endif
+#ifdef SO_BROADCAST
+			if (sockopts & STREAM_SOCKOP_SO_BROADCAST) {
+				setsockopt(sock, SOL_SOCKET, SO_BROADCAST, (char*)&sockoptval, sizeof(sockoptval));
 			}
 #endif
 
@@ -487,7 +505,7 @@ php_socket_t php_network_bind_socket_to_local_addr(const char *host, unsigned po
 		*error_code = err;
 	}
 	if (error_string) {
-		*error_string = php_socket_strerror(err, NULL, 0);
+		*error_string = php_socket_error_str(err);
 	}
 
 bound:
@@ -499,7 +517,7 @@ bound:
 }
 /* }}} */
 
-PHPAPI int php_network_parse_network_address_with_port(const char *addr, long addrlen, struct sockaddr *sa, socklen_t *sl TSRMLS_DC)
+PHPAPI int php_network_parse_network_address_with_port(const char *addr, zend_long addrlen, struct sockaddr *sa, socklen_t *sl)
 {
 	char *colon;
 	char *tmp;
@@ -508,7 +526,7 @@ PHPAPI int php_network_parse_network_address_with_port(const char *addr, long ad
 	struct sockaddr_in *in4 = (struct sockaddr_in*)sa;
 	struct sockaddr **psal;
 	int n;
-	char *errstr = NULL;
+	zend_string *errstr = NULL;
 #if HAVE_IPV6
 	struct sockaddr_in6 *in6 = (struct sockaddr_in6*)sa;
 #endif
@@ -550,12 +568,12 @@ PHPAPI int php_network_parse_network_address_with_port(const char *addr, long ad
 	}
 
 	/* looks like we'll need to resolve it */
-	n = php_network_getaddresses(tmp, SOCK_DGRAM, &psal, &errstr TSRMLS_CC);
+	n = php_network_getaddresses(tmp, SOCK_DGRAM, &psal, &errstr);
 
 	if (n == 0) {
 		if (errstr) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Failed to resolve `%s': %s", tmp, errstr);
-			STR_FREE(errstr);
+			php_error_docref(NULL, E_WARNING, "Failed to resolve `%s': %s", tmp, ZSTR_VAL(errstr));
+			zend_string_release(errstr);
 		}
 		goto out;
 	}
@@ -581,7 +599,7 @@ PHPAPI int php_network_parse_network_address_with_port(const char *addr, long ad
 	php_network_freeaddresses(psal);
 
 out:
-	STR_FREE(tmp);
+	efree(tmp);
 	return ret;
 }
 
@@ -590,11 +608,11 @@ PHPAPI void php_network_populate_name_from_sockaddr(
 		/* input address */
 		struct sockaddr *sa, socklen_t sl,
 		/* output readable address */
-		char **textaddr, long *textaddrlen,
+		zend_string **textaddr,
 		/* output address */
 		struct sockaddr **addr,
 		socklen_t *addrlen
-		TSRMLS_DC)
+		)
 {
 	if (addr) {
 		*addr = emalloc(sl);
@@ -613,7 +631,7 @@ PHPAPI void php_network_populate_name_from_sockaddr(
 				/* generally not thread safe, but it *is* thread safe under win32 */
 				buf = inet_ntoa(((struct sockaddr_in*)sa)->sin_addr);
 				if (buf) {
-					*textaddrlen = spprintf(textaddr, 0, "%s:%d",
+					*textaddr = strpprintf(0, "%s:%d",
 						buf, ntohs(((struct sockaddr_in*)sa)->sin_port));
 				}
 
@@ -623,7 +641,7 @@ PHPAPI void php_network_populate_name_from_sockaddr(
 			case AF_INET6:
 				buf = (char*)inet_ntop(sa->sa_family, &((struct sockaddr_in6*)sa)->sin6_addr, (char *)&abuf, sizeof(abuf));
 				if (buf) {
-					*textaddrlen = spprintf(textaddr, 0, "%s:%d",
+					*textaddr = strpprintf(0, "%s:%d",
 						buf, ntohs(((struct sockaddr_in6*)sa)->sin6_port));
 				}
 
@@ -637,13 +655,10 @@ PHPAPI void php_network_populate_name_from_sockaddr(
 					if (ua->sun_path[0] == '\0') {
 						/* abstract name */
 						int len = strlen(ua->sun_path + 1) + 1;
-						*textaddrlen = len;
-						*textaddr = emalloc(len + 1);
-						memcpy(*textaddr, ua->sun_path, len);
-						(*textaddr)[len] = '\0';
+						*textaddr = zend_string_init((char*)ua->sun_path, len, 0);
 					} else {
-						*textaddrlen = strlen(ua->sun_path);
-						*textaddr = estrndup(ua->sun_path, *textaddrlen);
+						int len = strlen(ua->sun_path);
+						*textaddr = zend_string_init((char*)ua->sun_path, len, 0);
 					}
 				}
 				break;
@@ -655,10 +670,10 @@ PHPAPI void php_network_populate_name_from_sockaddr(
 }
 
 PHPAPI int php_network_get_peer_name(php_socket_t sock,
-		char **textaddr, long *textaddrlen,
+		zend_string **textaddr,
 		struct sockaddr **addr,
 		socklen_t *addrlen
-		TSRMLS_DC)
+		)
 {
 	php_sockaddr_storage sa;
 	socklen_t sl = sizeof(sa);
@@ -666,19 +681,19 @@ PHPAPI int php_network_get_peer_name(php_socket_t sock,
 
 	if (getpeername(sock, (struct sockaddr*)&sa, &sl) == 0) {
 		php_network_populate_name_from_sockaddr((struct sockaddr*)&sa, sl,
-				textaddr, textaddrlen,
+				textaddr,
 				addr, addrlen
-				TSRMLS_CC);
+				);
 		return 0;
 	}
 	return -1;
 }
 
 PHPAPI int php_network_get_sock_name(php_socket_t sock,
-		char **textaddr, long *textaddrlen,
+		zend_string **textaddr,
 		struct sockaddr **addr,
 		socklen_t *addrlen
-		TSRMLS_DC)
+		)
 {
 	php_sockaddr_storage sa;
 	socklen_t sl = sizeof(sa);
@@ -686,9 +701,9 @@ PHPAPI int php_network_get_sock_name(php_socket_t sock,
 
 	if (getsockname(sock, (struct sockaddr*)&sa, &sl) == 0) {
 		php_network_populate_name_from_sockaddr((struct sockaddr*)&sa, sl,
-				textaddr, textaddrlen,
+				textaddr,
 				addr, addrlen
-				TSRMLS_CC);
+				);
 		return 0;
 	}
 	return -1;
@@ -700,19 +715,19 @@ PHPAPI int php_network_get_sock_name(php_socket_t sock,
  * using an optional timeout.
  * Returns the peer address in addr/addrlen (it will emalloc
  * these, so be sure to efree the result).
- * If you specify textaddr/textaddrlen, a text-printable
+ * If you specify textaddr, a text-printable
  * version of the address will be emalloc'd and returned.
  * */
 
 /* {{{ php_network_accept_incoming */
 PHPAPI php_socket_t php_network_accept_incoming(php_socket_t srvsock,
-		char **textaddr, long *textaddrlen,
+		zend_string **textaddr,
 		struct sockaddr **addr,
 		socklen_t *addrlen,
 		struct timeval *timeout,
-		char **error_string,
+		zend_string **error_string,
 		int *error_code
-		TSRMLS_DC)
+		)
 {
 	php_socket_t clisock = -1;
 	int error = 0, n;
@@ -732,9 +747,9 @@ PHPAPI php_socket_t php_network_accept_incoming(php_socket_t srvsock,
 
 		if (clisock != SOCK_ERR) {
 			php_network_populate_name_from_sockaddr((struct sockaddr*)&sa, sl,
-					textaddr, textaddrlen,
+					textaddr,
 					addr, addrlen
-					TSRMLS_CC);
+					);
 		} else {
 			error = php_socket_errno();
 		}
@@ -744,7 +759,7 @@ PHPAPI php_socket_t php_network_accept_incoming(php_socket_t srvsock,
 		*error_code = error;
 	}
 	if (error_string) {
-		*error_string = php_socket_strerror(error, NULL, 0);
+		*error_string = php_socket_error_str(error);
 	}
 
 	return clisock;
@@ -761,9 +776,9 @@ PHPAPI php_socket_t php_network_accept_incoming(php_socket_t srvsock,
 
 /* {{{ php_network_connect_socket_to_host */
 php_socket_t php_network_connect_socket_to_host(const char *host, unsigned short port,
-		int socktype, int asynchronous, struct timeval *timeout, char **error_string,
-		int *error_code, char *bindto, unsigned short bindport
-		TSRMLS_DC)
+		int socktype, int asynchronous, struct timeval *timeout, zend_string **error_string,
+		int *error_code, char *bindto, unsigned short bindport, long sockopts
+		)
 {
 	int num_addrs, n, fatal = 0;
 	php_socket_t sock;
@@ -774,7 +789,7 @@ php_socket_t php_network_connect_socket_to_host(const char *host, unsigned short
 	struct timeval limit_time, time_now;
 #endif
 
-	num_addrs = php_network_getaddresses(host, socktype, &psal, error_string TSRMLS_CC);
+	num_addrs = php_network_getaddresses(host, socktype, &psal, error_string);
 
 	if (num_addrs == 0) {
 		/* could not resolve address(es) */
@@ -844,7 +859,7 @@ php_socket_t php_network_connect_socket_to_host(const char *host, unsigned short
 					in4->sin_family = sa->sa_family;
 					in4->sin_port = htons(bindport);
 					if (!inet_aton(bindto, &in4->sin_addr)) {
-						php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid IP Address: %s", bindto);
+						php_error_docref(NULL, E_WARNING, "Invalid IP Address: %s", bindto);
 						goto skip_bind;
 					}
 					memset(&(in4->sin_zero), 0, sizeof(in4->sin_zero));
@@ -859,13 +874,14 @@ php_socket_t php_network_connect_socket_to_host(const char *host, unsigned short
 					in6->sin6_family = sa->sa_family;
 					in6->sin6_port = htons(bindport);
 					if (inet_pton(AF_INET6, bindto, &in6->sin6_addr) < 1) {
-						php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid IP Address: %s", bindto);
+						php_error_docref(NULL, E_WARNING, "Invalid IP Address: %s", bindto);
 						goto skip_bind;
 					}
 				}
 #endif
+
 				if (!local_address || bind(sock, local_address, local_address_len)) {
-					php_error_docref(NULL TSRMLS_CC, E_WARNING, "failed to bind to '%s:%d', system said: %s", bindto, bindport, strerror(errno));
+					php_error_docref(NULL, E_WARNING, "failed to bind to '%s:%d', system said: %s", bindto, bindport, strerror(errno));
 				}
 skip_bind:
 				if (local_address) {
@@ -874,10 +890,18 @@ skip_bind:
 			}
 			/* free error string received during previous iteration (if any) */
 			if (error_string && *error_string) {
-				efree(*error_string);
+				zend_string_release(*error_string);
 				*error_string = NULL;
 			}
 
+#ifdef SO_BROADCAST
+			{
+				int val = 1;
+				if (sockopts & STREAM_SOCKOP_SO_BROADCAST) {
+					setsockopt(sock, SOL_SOCKET, SO_BROADCAST, (char*)&val, sizeof(val));
+				}
+			}
+#endif
 			n = php_network_connect_socket(sock, sa, socklen, asynchronous,
 					timeout ? &working_timeout : NULL,
 					error_string, error_code);
@@ -992,6 +1016,7 @@ PHPAPI char *php_socket_strerror(long err, char *buf, size_t bufsize)
 		buf = estrdup(errstr);
 	} else {
 		strncpy(buf, errstr, bufsize);
+		buf[bufsize?(bufsize-1):0] = 0;
 	}
 	return buf;
 #else
@@ -1016,6 +1041,7 @@ PHPAPI char *php_socket_strerror(long err, char *buf, size_t bufsize)
 		buf = estrdup(sysbuf);
 	} else {
 		strncpy(buf, sysbuf, bufsize);
+		buf[bufsize?(bufsize-1):0] = 0;
 	}
 
 	if (free_it) {
@@ -1027,8 +1053,46 @@ PHPAPI char *php_socket_strerror(long err, char *buf, size_t bufsize)
 }
 /* }}} */
 
+/* {{{ php_socket_error_str */
+PHPAPI zend_string *php_socket_error_str(long err)
+{
+#ifndef PHP_WIN32
+	char *errstr;
+
+	errstr = strerror(err);
+	return zend_string_init(errstr, strlen(errstr), 0);
+#else
+	zend_string *ret;
+	char *sysbuf;
+	int free_it = 1;
+
+	if (!FormatMessage(
+				FORMAT_MESSAGE_ALLOCATE_BUFFER |
+				FORMAT_MESSAGE_FROM_SYSTEM |
+				FORMAT_MESSAGE_IGNORE_INSERTS,
+				NULL,
+				err,
+				MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
+				(LPTSTR)&sysbuf,
+				0,
+				NULL)) {
+		free_it = 0;
+		sysbuf = "Unknown Error";
+	}
+
+	ret = zend_string_init(sysbuf, strlen(sysbuf), 0);
+
+	if (free_it) {
+		LocalFree(sysbuf);
+	}
+
+	return ret;
+#endif
+}
+/* }}} */
+
 /* deprecated */
-PHPAPI php_stream *_php_stream_sock_open_from_socket(php_socket_t socket, const char *persistent_id STREAMS_DC TSRMLS_DC)
+PHPAPI php_stream *_php_stream_sock_open_from_socket(php_socket_t socket, const char *persistent_id STREAMS_DC)
 {
 	php_stream *stream;
 	php_netstream_data_t *sock;
@@ -1053,10 +1117,10 @@ PHPAPI php_stream *_php_stream_sock_open_from_socket(php_socket_t socket, const
 }
 
 PHPAPI php_stream *_php_stream_sock_open_host(const char *host, unsigned short port,
-		int socktype, struct timeval *timeout, const char *persistent_id STREAMS_DC TSRMLS_DC)
+		int socktype, struct timeval *timeout, const char *persistent_id STREAMS_DC)
 {
 	char *res;
-	long reslen;
+	zend_long reslen;
 	php_stream *stream;
 
 	reslen = spprintf(&res, 0, "tcp://%s:%d", host, port);
@@ -1069,20 +1133,22 @@ PHPAPI php_stream *_php_stream_sock_open_host(const char *host, unsigned short p
 	return stream;
 }
 
-PHPAPI int php_set_sock_blocking(int socketd, int block TSRMLS_DC)
+PHPAPI int php_set_sock_blocking(php_socket_t socketd, int block)
 {
 	int ret = SUCCESS;
-	int flags;
-	int myflag = 0;
 
 #ifdef PHP_WIN32
+	u_long flags;
+
 	/* with ioctlsocket, a non-zero sets nonblocking, a zero sets blocking */
 	flags = !block;
 	if (ioctlsocket(socketd, FIONBIO, &flags) == SOCKET_ERROR) {
 		ret = FAILURE;
 	}
 #else
-	flags = fcntl(socketd, F_GETFL);
+	int myflag = 0;
+	int flags = fcntl(socketd, F_GETFL);
+
 #ifdef O_NONBLOCK
 	myflag = O_NONBLOCK; /* POSIX version */
 #elif defined(O_NDELAY)
@@ -1102,10 +1168,9 @@ PHPAPI int php_set_sock_blocking(int socketd, int block TSRMLS_DC)
 
 PHPAPI void _php_emit_fd_setsize_warning(int max_fd)
 {
-	TSRMLS_FETCH();
 
 #ifdef PHP_WIN32
-	php_error_docref(NULL TSRMLS_CC, E_WARNING,
+	php_error_docref(NULL, E_WARNING,
 		"PHP needs to be recompiled with a larger value of FD_SETSIZE.\n"
 		"If this binary is from an official www.php.net package, file a bug report\n"
 		"at http://bugs.php.net, including the following information:\n"
@@ -1115,7 +1180,7 @@ PHPAPI void _php_emit_fd_setsize_warning(int max_fd)
 		"one time, in order to avoid seeing this error again at a later date.",
 		FD_SETSIZE, max_fd, (max_fd + 128) & ~127);
 #else
-	php_error_docref(NULL TSRMLS_CC, E_WARNING,
+	php_error_docref(NULL, E_WARNING,
 		"You MUST recompile PHP with a larger value of FD_SETSIZE.\n"
 		"It is set to %d, but you have descriptors numbered at least as high as %d.\n"
 		" --enable-fd-setsize=%d is recommended, but you may want to set it\n"
@@ -1191,9 +1256,95 @@ PHPAPI int php_poll2(php_pollfd *ufds, unsigned int nfds, int timeout)
 	}
 	return n;
 }
-
 #endif
 
+#if defined(HAVE_GETHOSTBYNAME_R)
+#ifdef HAVE_FUNC_GETHOSTBYNAME_R_6
+struct hostent * gethostname_re (const char *host,struct hostent *hostbuf,char **tmphstbuf,size_t *hstbuflen)
+{
+	struct hostent *hp;
+	int herr,res;
+
+	if (*hstbuflen == 0) {
+		*hstbuflen = 1024; 
+		*tmphstbuf = (char *)malloc (*hstbuflen);
+	}
+
+	while (( res = 
+		gethostbyname_r(host,hostbuf,*tmphstbuf,*hstbuflen,&hp,&herr))
+		&& (errno == ERANGE)) {
+		/* Enlarge the buffer. */
+		*hstbuflen *= 2;
+		*tmphstbuf = (char *)realloc (*tmphstbuf,*hstbuflen);
+	}
+
+	if (res != SUCCESS) {
+		return NULL;
+	}
+		
+	return hp;
+}
+#endif
+#ifdef HAVE_FUNC_GETHOSTBYNAME_R_5
+struct hostent * gethostname_re (const char *host,struct hostent *hostbuf,char **tmphstbuf,size_t *hstbuflen)
+{
+	struct hostent *hp;
+	int herr;
+
+	if (*hstbuflen == 0) {
+		*hstbuflen = 1024;
+		*tmphstbuf = (char *)malloc (*hstbuflen);
+	}
+
+	while ((NULL == ( hp = 
+		gethostbyname_r(host,hostbuf,*tmphstbuf,*hstbuflen,&herr)))
+		&& (errno == ERANGE)) {
+		/* Enlarge the buffer. */
+		*hstbuflen *= 2;
+		*tmphstbuf = (char *)realloc (*tmphstbuf,*hstbuflen);
+	}
+	return hp;
+}
+#endif
+#ifdef HAVE_FUNC_GETHOSTBYNAME_R_3
+struct hostent * gethostname_re (const char *host,struct hostent *hostbuf,char **tmphstbuf,size_t *hstbuflen)
+{
+	if (*hstbuflen == 0) {
+		*hstbuflen = sizeof(struct hostent_data);
+		*tmphstbuf = (char *)malloc (*hstbuflen);
+	} else {
+		if (*hstbuflen < sizeof(struct hostent_data)) {
+			*hstbuflen = sizeof(struct hostent_data);
+			*tmphstbuf = (char *)realloc(*tmphstbuf, *hstbuflen);
+		}
+	}
+	memset((void *)(*tmphstbuf),0,*hstbuflen);
+
+	if (SUCCESS != gethostbyname_r(host,hostbuf,(struct hostent_data *)*tmphstbuf)) {
+		return NULL;
+	}
+
+	return hostbuf;
+}
+#endif
+#endif
+
+PHPAPI struct hostent*	php_network_gethostbyname(char *name) {
+#if !defined(HAVE_GETHOSTBYNAME_R)
+	return gethostbyname(name);
+#else
+	if (FG(tmp_host_buf)) {
+		free(FG(tmp_host_buf));
+	}
+
+	FG(tmp_host_buf) = NULL;
+	FG(tmp_host_buf_len) = 0;
+
+	memset(&FG(tmp_host_info), 0, sizeof(struct hostent));
+
+	return gethostname_re(name, &FG(tmp_host_info), &FG(tmp_host_buf), &FG(tmp_host_buf_len));
+#endif
+}
 
 /*
  * Local variables:
